package xmlchecker;

import java.util.HashMap;
import java.util.Iterator;

import java.awt.Component;
import java.awt.Font;

/**
  Fontskalierung fr registrierte Komponenten
*/
public class FontScaler{
  private HashMap<Component, Integer> compinisize;
  private float factor;

/**
  Standardkonstruktor
*/
  public FontScaler(){
    this.compinisize = new HashMap<Component, Integer>();
    this.factor = 1.0f;
  }

/**
  Zu skalierende Komponente in Liste registrieren. Der Font wird mit der aktuellen Skalierung versehen
  @param   comp   die Komponente
*/
  public void add(Component comp){
    if(comp != null){
      this.compinisize.put(comp, new Integer(comp.getFont().getSize()));
      this.scaleComponent(comp);
    }
  }
/**
  Komponente aus der Liste entfernen
  @param   comp   die zu entfernende Komponente
*/
  public void remove(Component comp){
    this.compinisize.remove(comp);
  }
/**
  Alle Komponenten aus der Liste
  @return  die zu skalierenden Komponenten
*/
  public Component[] getComponents(){
    return (Component[])this.compinisize.keySet().toArray();
  }

/**
  Eine Komponente aus der Liste mit der aktuellen Skalierung versehen
  @param  comp   die zu skalierende Komponente
*/
  public void scaleComponent(Component comp){
    Integer inisize = this.compinisize.get(comp);
    if(inisize != null){
      comp.setFont(comp.getFont().deriveFont(inisize.intValue() * factor));
    }
  }
/**
  Alle Komponente aus der Liste mit der angegebenen Skalierung versehen
  @param  factor   der Skalierungsfaktor (>0). Die Angabe von 1.0f skaliert auf die Gre bei Registrierung
*/
  public void scale(float factor){
    if(0 < factor){
      this.factor = factor;
      Iterator<Component> comps = this.compinisize.keySet().iterator();
      while(comps.hasNext()){
        this.scaleComponent(comps.next());
      }
    }
  }
}