package xmlchecker.sax;

import java.net.URI;
import java.net.URISyntaxException;

import javax.swing.JOptionPane;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;

import xmlchecker.XMLTreeNode;

public class SAXCheckerContentHandler implements ContentHandler{
  private Locator docLocator = null;
  private XMLTreeNode rootNode = null, parentNode = null;
  private boolean namespaceAware;
  private StringBuffer charbuffer;

  public SAXCheckerContentHandler(XMLTreeNode root, boolean namespaceAware){
    this.rootNode = root;
    this.namespaceAware = namespaceAware;
    charbuffer = new StringBuffer();
  }

  public void setDocumentLocator(Locator locator){
    this.docLocator = locator;
  }

  public void startDocument(){
    this.rootNode.removeAllChildren();
    this.rootNode.setText("XML-Dokument");
    this.rootNode.setType(XMLTreeNode.DOCUMENT);
    this.parentNode = rootNode;
  }
  public void endDocument(){}

  public void processingInstruction(String target, String data){
    XMLTreeNode node = new XMLTreeNode("Processing Instruction", XMLTreeNode.PI);
    this.parentNode.add(node);
    node.add(new XMLTreeNode(target, XMLTreeNode.UNDEFINED));
    node.add(new XMLTreeNode(data, XMLTreeNode.UNDEFINED));
  }

  public void startElement(String nsuri, String lName, String qName, Attributes attrs){
    if(charbuffer.length()>0){
      parentNode.add(new XMLTreeNode(new String(charbuffer), XMLTreeNode.CHARACTERS));
      charbuffer = new StringBuffer();
    }
    String name = qName;
    if(this.namespaceAware && lName.length()>0){
      if(nsuri.length()>0){
        name = "{" + nsuri + "}" + lName;
      }
    }
    XMLTreeNode node = new XMLTreeNode(name, XMLTreeNode.ELEMENT);
    parentNode.add(node);
    parentNode = node;
    for(int i=0; i<attrs.getLength(); i++){
      name = attrs.getQName(i);
      if(this.namespaceAware && attrs.getLocalName(i).length()>0){
        if(attrs.getURI(i).length()>0){
          name = "{" + attrs.getURI(i) + "}" + attrs.getLocalName(i);
        } else {
          name = attrs.getLocalName(i);
        }
        
        if("http://www.w3.org/2001/XMLSchema-instance".equals(attrs.getURI(i))) {
          String lname = attrs.getLocalName(i);
          String value = attrs.getValue(i);
          if("noNamespaceSchemaLocation".equals(lname)) {
            try {
              new URI(value);
            } catch (URISyntaxException e) {
                JOptionPane.showMessageDialog(null, "URI fr anonymen Namensraum: " + e.getMessage(), "Fehler bei Attribut noNamespaceSchemaLocation", JOptionPane.ERROR_MESSAGE);
			}
          }
          if("schemaLocation".equals(lname)) {
            String[] vals = value.split("\\s+");
            for(int ivals = 0; ivals < vals.length; ivals+= 2) {
              String schemaid = vals[ivals];
              String schemauri = vals[ivals + 1];
              try {
                  new URI(schemauri);
              } catch (URISyntaxException e) {
                  JOptionPane.showMessageDialog(null, "URI fr Namensraum "+ schemaid + ": " + e.getMessage(), "Fehler bei Attribut schemaLocation", JOptionPane.ERROR_MESSAGE);
              }
            }
          }
        }
      }
      parentNode.add(new XMLTreeNode(name + "=\"" + attrs.getValue(i) + "\"", XMLTreeNode.ATTRIBUTE));
    }
  }
  public void endElement(String nsuri, String lName, String qName){
    if(charbuffer.length()>0){
      parentNode.add(new XMLTreeNode(new String(charbuffer), XMLTreeNode.CHARACTERS));
      charbuffer = new StringBuffer();
    }
    parentNode = (XMLTreeNode)parentNode.getParent();
  }

  public void startPrefixMapping(String prefix, String nsuri){}
  public void endPrefixMapping(String prefix){}

  public void ignorableWhitespace(char[] docsec, int start, int length){}

  public void characters(char[] docsec, int start, int length){
      charbuffer.append(docsec, start, length);
  }

  public void skippedEntity(String entityRef){}
}